/*
(C) ALVIDI 24.4.2011

Disclaimer
----------
Wir haften nicht fr die Eignung der Software fr einen bestimmten, vom Kunden beabsichtigten Verwendungszweck.
We are not liable for the problems, which occur during incorrect usage of our products.

Wir bernehmen keine Haftung fr die Fehler, die im Laufe der Nutzung auftreten knnen.
We don't take liability for mistakes, which may occur while using our products.

Wir bernehmen keine Verantwortung fr mgliche Schden, die bei der Nutzung des Programms entstehen knnen.
We don't take responsibility for possible damages, which may happen while using our products.

Dieses Programm ist freie Software. Sie knnen es verffentlichen, weitergeben und/oder modifizieren.
This program is freeware. You may publish it, refer and/or modify.

Diese Software ist kompatibel mit dem AVR32 Board http://alvidi.de/avr32_board.html
This software is compatible with the AVR32 Board http://alvidi.de/avr32_board_en.html
*/

#ifndef KSZ8041TL_H_INCLUDED
#define KSZ8041TL_H_INCLUDED

#include "macb.h"
#include "delay.h"

/* Phy Identifier for the KSZ8041TL */
#define EXTPHY_PHY_ID               0x00221510



/* Extended registers for KSZ8041TL */
#define PHY_ICSR			0x1B		//!< Interrupt Control/Status reg
#define LINK_MD_CS			0x1D		//!< LinkMD Control/Status reg
#define PHY_PHYCR           0x1F        //!< Phy CTRL reg

/* Interrupt Control/Status Register. */
#define ICSR_JAE_INT			0x8000	//!< Enable Jabber Interrupt
#define ICSR_REE_INT			0x4000	//!< Enable Receive Error Interrupt
#define ICSR_PRE_INT			0x2000	//!< Enable Page Received Interrupt
#define ICSR_PDFE_INT			0x1000	//!< Enable Parallel Detect Fault Interrupt
#define ICSR_LPAE_INT			0x0800	//!< Enable Link Partner Acknowledge Interrupt
#define ICSR_LDE_INT			0x0400	//!< Enable Link Down Interrupt
#define ICSR_RFE_INT			0x0200	//!< Enable Remote Fault Interrupt
#define ICSR_LUE_INT			0x0100	//!< Enable Link Up Interrupt

#define ICSR_JAO_INT			0x0080	//!< Jabber occurred
#define ICSR_REO_INT			0x0040	//!< Receive Error occurred
#define ICSR_PRO_INT			0x0020	//!< Page Received occurred
#define ICSR_PDFO_INT			0x0010	//!< Parallel Detect Fault occurred
#define ICSR_LPAO_INT			0x0008	//!< Link Partner Acknowledge occurred
#define ICSR_LDO_INT			0x0004	//!< Link Down occurred
#define ICSR_RFO_INT			0x0002	//!< Remote Fault occurred
#define ICSR_LUO_INT			0x0001	//!< Link Up occurred

/* LinkMD Control/Status Register. */
#define LINK_MD_CS_CDTE			0x8000	//!< Enable cable diagnostic test
#define LINK_MD_CS_CDTRH		0x4000	//!< Cable diagnostic test result high bit
#define LINK_MD_CS_CDTRL		0x2000	//!< Cable diagnostic test result low bit
#define LINK_MD_CS_CFC_MASK		0x01FF	//!< Cable Fault Counter

/* PHY Control Register. */
#define PHYCR_LED_MODE_MASK		 0xC000	 //!< LED mode
#define PHYCR_POL				 0x2000	 //!< Polarity
#define PHYCR_FAR_END			 0x1000	 //!< Far-End fault Detect
#define PHYCR_MDI_X_STATE		 0x0800  //!< MDI/MDI-X state
#define PHYCR_REM_LOOP			 0x0080	 //!< Remote loopback

#define PHYCR_HP_MDIX			 0x8000	 //!< HP_MDIX
#define PHYCR_MDI_X_SELECT		 0x4000	 //!< MDI/MDI-X select
#define PHYCR_MDIX_EN            0x2000  //!< Enable Auto MDIX
#define PHYCR_ENERGY_DETECT		 0x1000	 //!< Energy Detect
#define PHYCR_FORCE_LINK         0x0800  //!< Force Link
#define PHYCR_PWR_SAVE			 0x0400  //!< Power saving
#define PHYCR_INT_LVL			 0x0200	 //!< Interrupt Level
#define PHYCR_ENABLE_JAB		 0x0100  //!< Ennable Jabber
#define PHYCR_AUTONEG_RDY		 0x0080  //!< Auto-Negotiation complete
#define PHYCR_ENABLE_PAUSE		 0x0040  //!< Enable Pause (flow control)
#define PHYCR_PHY_ISOLATE		 0x0020  //!< PHY Isolate
#define PHYCR_OPERATE_MODE_MASK	 0x001C	 //!< Operation Mode Indication
#define PHYCR_ENABLE_SQE_TEST	 0x0002	 //!< Enable SQE test
#define PHYCR_DISABLE_DATASCR	 0x0001  //!< Disable data scrambling

/* register from dp83848 to ksz8041tl*/
#define PHY_MICR  				 PHY_ICSR
#define PHY_MISR  				 PHY_ICSR
#define MICR_INTEN  			 (ICSR_LDE_INT | ICSR_LUE_INT)
#define MICR_INTOE  			 0
#define MISR_LINK_INT_EN		 (ICSR_LDO_INT | ICSR_LUO_INT)

/* Generate an Hardware Reset on the Phy. */
static inline void ethernet_phy_hw_reset(void)
{
	 delay_init(ETHERNET_CONF_SYSTEM_CLOCK);
	 gpio_clr_gpio_pin(AVR32_PIN_PX22);
	 delay_ms(100);
	 gpio_set_gpio_pin(AVR32_PIN_PX22);
	 delay_ms(1000);
}


/* Generate a Software Reset on the Phy. */
static inline void ethernet_phy_sw_reset(volatile avr32_macb_t *macb)
{
#if false
  volatile unsigned long config;

  // read Control register
  config = ulReadMDIO(macb, PHY_BMCR);
  config |= BMCR_RESET;
  // update ctrl register
  vWriteMDIO(macb, PHY_BMCR, config);

  // loop while link status isn't OK
  do {
    config = ulReadMDIO(macb, PHY_BMCR);
  }while (!(config & BMCR_RESET));
#endif
}

/* Setup the Phy in RMII mode, by ksz8041tl via hardware connection. */
static inline void ethernet_phy_setup_rmii(volatile avr32_macb_t *macb)
{}


/* Setup auto-negociation for the Phy. */
static inline void ethernet_phy_setup_auto_negociation(volatile avr32_macb_t *macb, volatile unsigned long *config)
{
    volatile unsigned long phy_ctrl;

    // read Phy Control register
    phy_ctrl = ulReadMDIO(macb, PHY_PHYCR);

#if ETHERNET_CONF_AN_ENABLE

    #if ETHERNET_CONF_AUTO_CROSS_ENABLE
      // enable Auto MDIX
      phy_ctrl &= ~PHYCR_MDIX_EN;
    #else
      // disable Auto MDIX
      phy_ctrl |= PHYCR_MDIX_EN;
    #endif
        
    // reset auto-negociation capability
    *config |= (BMCR_ANRESTART | BMCR_ANENABLE);
#endif

    // update Phy ctrl register
    vWriteMDIO(macb, PHY_PHYCR, phy_ctrl);
}


#endif /* KSZ8041TL_H_INCLUDED */
