/*
(C) ALVIDI 22.7.2012
 
Disclaimer
----------
Wir haften nicht fr die Eignung der Software fr einen bestimmten, vom Kunden beabsichtigten Verwendungszweck.
We are not liable for the problems, which occur during incorrect usage of our products.

Wir bernehmen keine Haftung fr die Fehler, die im Laufe der Nutzung auftreten knnen.
We don't take liability for mistakes, which may occur while using our products.

Wir bernehmen keine Verantwortung fr mgliche Schden, die bei der Nutzung des Programms entstehen knnen.
We don't take responsibility for possible damages, which may happen while using our products.

Dieses Programm ist freie Software. Sie knnen es verffentlichen, weitergeben und/oder modifizieren. 
This program is freeware. You may publish it, refer and/or modify.

Diese Software ist kompatibel mit dem AVR32 Modul mit EEPROM http://alvidi.de/avr32_modul_bmb.html
This software is compatible with the AVR32 Module with EEPROM http://alvidi.de/avr32_module_bmb.html
*/

#include "twi.h"
#include "gpio.h"
#include "eeprom.h"
#include "board.h"
/****************************************************************************************************************/
void eeprom_init(unsigned int f_cpu)
{
twi_options_t opt;

gpio_enable_module_pin(EEPROM_TWI_SDA_PIN, EEPROM_TWI_SDA_FUNCTION);
gpio_enable_module_pin(EEPROM_TWI_SCL_PIN, EEPROM_TWI_SCL_FUNCTION);

opt.pba_hz = f_cpu;
opt.speed = EEPROM_SPEED;
opt.chip = CHIP_ADDRESS;

twi_master_init(&AVR32_TWI, &opt);
}
/****************************************************************************************************************/
int eeprom_write_byte(unsigned int addr, unsigned char sign)
{
char data[1]={0};
twi_package_t packet;

data[0]=sign;

packet.chip = CHIP_ADDRESS;
packet.addr_length = ADDRESS_LENGTH;

packet.addr[0] = addr >> 8;
packet.addr[1] = addr;

packet.buffer = data;
packet.length = 1;

if(addr < EEPROM_SIZE)
	while(twi_master_write(&AVR32_TWI, &packet)!=TWI_SUCCESS);

if(eeprom_read_byte(addr) == sign)
  return (EEPROM_SUCCESS);
else
  return (EEPROM_ERROR);
}
/****************************************************************************************************************/
int eeprom_write_block(unsigned int addr, char *data, unsigned int length)
{
unsigned int i;
twi_package_t packet;
char data_rec[EEPROM_PAGE_SIZE]={0};

data[length]=0;

if(length < EEPROM_PAGE_SIZE && (addr+length) < EEPROM_SIZE)
  {
  packet.chip = CHIP_ADDRESS;
  packet.addr_length = ADDRESS_LENGTH;

  packet.addr[0] = addr >> 8;
  packet.addr[1] = addr;
  
  packet.buffer = data;
  packet.length = length;

  while(twi_master_write(&AVR32_TWI, &packet)!=TWI_SUCCESS);

  eeprom_read_block(addr, data_rec, length);

  for(i=0; i<length; i++)
    if(data_rec[i]!=data[i])
      return(EEPROM_ERROR);
  }
else
  {
  return(EEPROM_ERROR);
  }

return (EEPROM_SUCCESS);
}
/****************************************************************************************************************/
int eeprom_read_byte(unsigned int addr)
{
char data[1]={0};
twi_package_t packet;

packet.chip = CHIP_ADDRESS;
packet.addr_length = ADDRESS_LENGTH;

packet.addr[0] = addr >> 8;
packet.addr[1] = addr;

packet.buffer = data;
packet.length = 1;

if(addr < EEPROM_SIZE)
	while(twi_master_read(&AVR32_TWI, &packet)!=TWI_SUCCESS);

return (*data);
}
/****************************************************************************************************************/
void eeprom_read_block(unsigned int addr, char *data, unsigned int length)
{
twi_package_t packet;

packet.chip = CHIP_ADDRESS ;
packet.addr_length = ADDRESS_LENGTH;

packet.addr[0] = addr >> 8;
packet.addr[1] = addr;

packet.buffer = data;
packet.length = length;

if((addr+length) < EEPROM_SIZE)
	while(twi_master_read(&AVR32_TWI, &packet)!=TWI_SUCCESS);

data[length]=0;
}
/****************************************************************************************************************/
